@@ -1,21 +1,20 @@
 /*
- *  Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
- *  Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
- *  Copyright © 2007, 2008, 2009 Christian Persch
+ * Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
+ * Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
+ * Copyright © 2007, 2008, 2009 Christian Persch
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope window it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -36,10 +35,6 @@
 #else
 #include <hildon/hildon-banner.h>
 #endif
-#ifdef HAVE_MAEMO_5
-#include <hildon/hildon-gtk.h>
-#include <hildon/hildon-pannable-area.h>
-#endif
 #endif /* HAVE_HILDON */
 
 #include <libgames-support/games-card-theme.h>
@@ -55,11 +50,21 @@
 #include <libgames-support/games-atk-utils.h>
 #endif
 
+#ifdef HAVE_CLUTTER
+#include "ar-clutter-embed.h"
+#include "ar-style.h"
+#include "baize.h"
 #include "board.h"
+#else
+#include "board-noclutter.h"
+#endif
+
 #include "conf.h"
 #include "game.h"
 #include "stats-dialog.h"
 #include "util.h"
+#include "ar-fullscreen-button.h"
+#include "ar-game-chooser.h"
 
 #include "window.h"
 
@@ -81,12 +86,24 @@
 /* The maximum number of recent games saved */
 #define MAX_RECENT 5
 
-#define SELECT_GAME_DIALOG_MIN_HEIGHT (256)
-
 #if !GLIB_CHECK_VERSION (2, 16, 0)
 #define C_(context, string) (_(string))
 #endif
 
+/* On maemo5, the toolbar doesn't have enough space to show a
+ * significant amount of text, so we always use the banner here.
+ */
+#if defined(HAVE_HILDON) && (defined(HAVE_MAEMO_3) || defined(HAVE_MAEMO_4))
+#define MAEMO_TOOLBAR_BANNER
+#endif
+
+/* On maemo5, there's no hardware key to exit the fullscreen mode. So we show
+ * an overlay button to restore normal mode, if the toolbar is hidden too.
+ */
+#if defined(HAVE_HILDON) && defined(HAVE_MAEMO_5)
+#define LEAVE_FULLSCREEN_BUTTON
+#endif
+
 /* define this to enable a debug menu */
 /* #undef ENABLE_DEBUG_UI */
 
@@ -103,7 +120,7 @@ enum
 #ifdef HAVE_HILDON
   ACTION_ACCEL_UNDO_MOVE,
 #endif
-#ifndef HAVE_MAEMO
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
   ACTION_LEAVE_FULLSCREEN,
 #endif
   LAST_ACTION
@@ -112,21 +129,30 @@ enum
 struct _AisleriotWindowPrivate
 {
   AisleriotGame *game;
+#ifdef HAVE_CLUTTER
+  ArClutterEmbed *board;
+  ArStyle *board_style;
+  ClutterActor *baize_actor;
+  ClutterActor *board_actor;
+#else
   AisleriotBoard *board;
+#endif
 
   GamesCardThemes *theme_manager;
   GamesCardTheme *theme;
 
 #ifdef HAVE_HILDON
-  GtkLabel *game_message_label;
   guint game_message_hash;
+#ifdef MAEMO_TOOLBAR_BANNER
+  GtkLabel *game_message_label;
+#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   GtkStatusbar *statusbar;
   guint game_message_id;
   GtkWidget *score_box;
   GtkWidget *score_label;
   GtkWidget *clock;
-#endif
+#endif /* HAVE_HILDON */
 
   GtkUIManager *ui_manager;
   GtkActionGroup *action_group;
@@ -146,20 +172,26 @@ struct _AisleriotWindowPrivate
   GtkWidget *game_over_dialog;
   GtkWidget *game_choice_dialog;
   AisleriotStatsDialog *stats_dialog;
-  GtkListStore *game_choice_store;
-  GtkTreeSelection *game_choice_selection;
 
 #ifndef HAVE_HILDON
   GtkWidget *hint_dialog;
 #endif
 
+#ifdef LEAVE_FULLSCREEN_BUTTON
+  GtkWidget *fullscreen_button;
+#endif
+
   guint load_idle_id;
 
+#ifndef HAVE_CLUTTER
   guint use_pixbuf_drawing : 1;
+#endif
+
   guint changing_game_type : 1;
   guint freecell_mode : 1;
   guint toolbar_visible : 1;
   guint statusbar_visible : 1;
+  guint fullscreen : 1;
 };
 
 enum {
@@ -167,209 +199,11 @@ enum {
   OPTION_RADIOMENU
 };
 
-/* Game choice dialogue */
-
 enum {
-  COL_NAME,
-  COL_GAME_FILE
+  PROP_0,
+  PROP_FREECELL_MODE
 };
 
-static void
-select_game_row_activated_cb (GtkWidget *widget,
-                              GtkTreePath *path,
-                              GtkTreeViewColumn *column,
-                              GtkDialog *dialog)
-{
-  /* Handle a double click by faking a click on the OK button. */
-  gtk_dialog_response (dialog, GTK_RESPONSE_OK);
-}
-
-static void
-select_game_response_cb (GtkWidget *dialog,
-                         int response,
-                         AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  char *game_file = NULL;
-
-  if (response == GTK_RESPONSE_OK &&
-      gtk_tree_selection_get_selected (priv->game_choice_selection, &model, &iter)) {
-    gtk_tree_model_get (model, &iter,
-                        COL_GAME_FILE, &game_file,
-                        -1);
-    g_assert (game_file != NULL);
-
-    aisleriot_window_set_game (window, game_file, 0);
-
-    /* We'll store the new game in conf when it's been loaded successfully */
-
-    g_free (game_file);
-  }
-
-  gtk_widget_destroy (dialog);
-}
-
-static void
-select_game_cb (GtkAction *action,
-                AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkWidget *dialog;
-  GtkListStore *list;
-  GtkWidget *list_view;
-  GtkTreeSelection *selection;
-  GtkWidget *scrolled_window;
-  GtkTreeViewColumn *column;
-  GtkCellRenderer *renderer;
-  GtkWidget *hbox;
-  GtkTreeIter current_iter, selection_iter;
-  gboolean current_iter_set = FALSE;
-  const char *current_game_file;
-  const char *games_dir;
-  GDir *dir;
-
-  if (priv->game_choice_dialog) {
-    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
-    return;
-  }
-
-  priv->game_choice_store = list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-
-#ifdef HAVE_MAEMO_5
-  list_view = hildon_gtk_tree_view_new_with_model (HILDON_UI_MODE_NORMAL, GTK_TREE_MODEL (list));
-#else
-  list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
-#endif
-  g_object_unref (list);
-
-  current_game_file = aisleriot_game_get_game_file (priv->game);
-
-  games_dir = games_runtime_get_directory (GAMES_RUNTIME_GAME_GAMES_DIRECTORY);
-
-  dir = g_dir_open (games_dir, 0, NULL);
-  if (dir != NULL) {
-    const char *game_file;
-
-    while ((game_file = g_dir_read_name (dir)) != NULL) {
-      char *game_name;
-      GtkTreeIter iter;
-
-      if (!g_str_has_suffix (game_file, ".scm") ||
-          strcmp (game_file, "sol.scm") == 0)
-        continue;
-
-      game_name = games_filename_to_display_name (game_file);
-
-      gtk_list_store_insert_with_values (GTK_LIST_STORE (list), &iter,
-                                         -1,
-                                         COL_NAME, game_name,
-                                         COL_GAME_FILE, game_file,
-                                         -1);
-
-      if (current_game_file &&
-          strcmp (current_game_file, game_file) == 0) {
-        current_iter = iter;
-        current_iter_set = TRUE;
-      }
-
-      g_free (game_name);
-    }
-
-    g_dir_close (dir);
-  }
-
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list),
-                                        0, GTK_SORT_ASCENDING);
-
-  dialog = gtk_dialog_new_with_buttons (_("Select Game"),
-                                        GTK_WINDOW (window),
-                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        _("_Select"), GTK_RESPONSE_OK,
-                                        NULL);
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-                      hbox, TRUE, TRUE, 0);
-  g_signal_connect (list_view, "row-activated",
-                    G_CALLBACK (select_game_row_activated_cb), dialog);
-
-  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list_view), FALSE);
-
-  renderer = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (NULL,
-                                                     renderer,
-                                                     "text", COL_NAME, NULL);
-
-  gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), column);
-
-  priv->game_choice_selection = selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
-  gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
-
-#ifdef HAVE_MAEMO_5
-  scrolled_window = hildon_pannable_area_new ();
-#else
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
-                                       GTK_SHADOW_IN);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-#endif /* HAVE_MAEMO_5 */
-
-  gtk_container_add (GTK_CONTAINER (scrolled_window), list_view);
-
-  gtk_box_pack_end (GTK_BOX (hbox), scrolled_window, TRUE, TRUE, 0);
-
-  /* Set initial focus to the list view. Otherwise, if you press Up/Down key,
-   * the selection jumps to the first entry in the list as the view gains
-   * focus.
-   */
-  gtk_widget_grab_focus (list_view);
-
-  gtk_widget_show_all (hbox);
-
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (select_game_response_cb), window);
-
-  priv->game_choice_dialog = dialog;
-  g_signal_connect (dialog, "destroy",
-                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
-
-  gtk_widget_set_size_request (scrolled_window, -1, SELECT_GAME_DIALOG_MIN_HEIGHT);
-  gtk_window_present (GTK_WINDOW (dialog));
-
-  /* Select the row corresponding to the currently loaded game,
-   * and scroll to it.
-   */
-  if (current_iter_set) {
-    gtk_tree_selection_select_iter (selection, &current_iter);
-  }
-
-  if (gtk_tree_selection_get_selected (selection, NULL, &selection_iter)) {
-    GtkTreePath *path;
-
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (list), &selection_iter);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_view), path, NULL,
-                                  TRUE,
-				  0.5, 0.0);
-    gtk_tree_path_free (path);
-  }
-}
-
 /* Game Over dialogue */
 
 enum {
@@ -541,7 +375,7 @@ new_game_cb (GtkAction *action,
   aisleriot_game_new_game (priv->game, NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (priv->board));
-};
+}
 
 static void
 undo_cb (GtkAction *action,
@@ -550,7 +384,11 @@ undo_cb (GtkAction *action,
   AisleriotWindowPrivate *priv = window->priv;
 
   /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_undo_move (priv->game);
 }
@@ -561,8 +399,11 @@ redo_cb (GtkAction *action,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
-  /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_redo_move (priv->game);
 }
@@ -666,6 +507,24 @@ restart_game (GtkAction *action,
   aisleriot_game_restart_game (priv->game);
 };
 
+static void
+select_game_cb (GtkAction *action,
+                AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  if (priv->game_choice_dialog) {
+    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+    return;
+  }
+
+  priv->game_choice_dialog = ar_game_chooser_new (window);
+  g_signal_connect (priv->game_choice_dialog, "destroy",
+                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
+
+  gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+}
+
 static void
 help_general_cb (GtkAction *action,
                  AisleriotWindow *window)
@@ -997,6 +856,7 @@ debug_choose_seed_cb (GtkAction *action,
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
+#ifndef HAVE_CLUTTER
 static void
 debug_pixbuf_drawing_cb (GtkToggleAction *action,
                          AisleriotWindow *window)
@@ -1007,9 +867,35 @@ debug_pixbuf_drawing_cb (GtkToggleAction *action,
   active = gtk_toggle_action_get_active (action);
   aisleriot_board_set_pixbuf_drawing (priv->board, active);
 }
+#endif /* !HAVE_CLUTTER */
 
 #endif /* ENABLE_DEBUG_UI */
 
+static void
+set_fullscreen_button_active (AisleriotWindow *window)
+{
+#ifdef LEAVE_FULLSCREEN_BUTTON
+  AisleriotWindowPrivate *priv = window->priv;
+  gboolean active;
+
+  active = priv->fullscreen && !priv->toolbar_visible;
+  if (!active) {
+    if (priv->fullscreen_button != NULL) {
+      ar_fullscreen_button_set_active (AR_FULLSCREEN_BUTTON (priv->fullscreen_button), FALSE);
+    }
+
+    return;
+  }
+
+  if (active && priv->fullscreen_button == NULL) {
+    priv->fullscreen_button = ar_fullscreen_button_new (GTK_WINDOW (window),
+                                                        GTK_CORNER_TOP_RIGHT);
+  }
+
+  ar_fullscreen_button_set_active (AR_FULLSCREEN_BUTTON (priv->fullscreen_button), TRUE);
+#endif /* LEAVE_FULLSCREEN_BUTTON */
+}
+
 static void
 toolbar_toggled_cb (GtkToggleAction *action,
                     AisleriotWindow *window)
@@ -1024,6 +910,8 @@ toolbar_toggled_cb (GtkToggleAction *action,
   priv->toolbar_visible = state != FALSE;
 
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_SHOW_TOOLBAR), state);
+
+  set_fullscreen_button_active (window);
 }
 
 #ifndef HAVE_HILDON
@@ -1055,15 +943,19 @@ set_fullscreen_actions (AisleriotWindow *window,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
+  priv->fullscreen = is_fullscreen;
+
 #ifndef HAVE_MAEMO
   g_object_set (priv->main_menu, "visible", !is_fullscreen, NULL);
+#endif /* HAVE_MAEMO */
 
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
   gtk_action_set_visible (priv->action[ACTION_LEAVE_FULLSCREEN], is_fullscreen);
   g_object_set (gtk_ui_manager_get_widget (priv->ui_manager, "/Toolbar/LeaveFullscreenSep"),
                 "visible", is_fullscreen,
                 "draw", FALSE,
                 NULL);
-#endif
+#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (priv->action[ACTION_FULLSCREEN]),
                                 is_fullscreen);
@@ -1085,7 +977,7 @@ fullscreen_toggled_cb (GtkToggleAction *action,
   }
 }
 
-#ifndef HAVE_MAEMO
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
 
 static void
 leave_fullscreen_cb (GtkAction *action,
@@ -1094,7 +986,7 @@ leave_fullscreen_cb (GtkAction *action,
   gtk_window_unfullscreen (window);
 }
 
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 
 static void
 clickmove_toggle_cb (GtkToggleAction *action,
@@ -1106,7 +998,11 @@ clickmove_toggle_cb (GtkToggleAction *action,
   click_to_move = gtk_toggle_action_get_active (action);
 
   aisleriot_game_set_click_to_move (priv->game, click_to_move);
+#ifdef HAVE_CLUTTER
+  ar_style_set_click_to_move (priv->board_style, click_to_move);
+#else
   aisleriot_board_set_click_to_move (priv->board, click_to_move);
+#endif
   
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_CLICK_TO_MOVE), click_to_move);
 }
@@ -1139,7 +1035,11 @@ animations_toggle_cb (GtkToggleAction *action,
 
   enabled = gtk_toggle_action_get_active (action);
 
+#ifdef HAVE_CLUTTER
+  ar_style_set_enable_animations (priv->board_style, enabled);
+#else
   aisleriot_board_set_animation_mode (priv->board, enabled);
+#endif
   
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
 }
@@ -1523,7 +1423,11 @@ aisleriot_window_take_card_theme (AisleriotWindow *window,
   }
 #endif /* GTK+ 2.10.0 */
 
+#ifdef HAVE_CLUTTER
+  ar_style_set_card_theme (priv->board_style, theme);
+#else
   aisleriot_board_set_card_theme (priv->board, theme);
+#endif
 }    
 
 static void
@@ -1843,7 +1747,10 @@ game_type_changed_cb (AisleriotGame *game,
   g_object_set (priv->action[ACTION_HELP_GAME], "label", game_name, NULL);
   g_object_set (priv->action[ACTION_OPTIONS_MENU], "label", game_name, NULL);
 
-  gtk_window_set_title (GTK_WINDOW (window), game_name);
+  /* In freecell mode, we've already set the title to something different */
+  if (!priv->freecell_mode) {
+    gtk_window_set_title (GTK_WINDOW (window), game_name);
+  }
 
   g_free (game_name);
 
@@ -1861,7 +1768,9 @@ game_type_changed_cb (AisleriotGame *game,
   gtk_action_set_visible (priv->action[ACTION_DEAL], dealable);
 
 #ifdef HAVE_HILDON
+#ifdef MAEMO_TOOLBAR_BANNER
   gtk_label_set_text (priv->game_message_label, NULL);
+#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
@@ -1869,7 +1778,7 @@ game_type_changed_cb (AisleriotGame *game,
 
   show_scores = (features & FEATURE_SCORE_HIDDEN) == 0;
   g_object_set (priv->score_box, "visible", show_scores, NULL);
-#endif
+#endif /* HAVE_HILDON */
 
   priv->changing_game_type = FALSE;
 }
@@ -1878,17 +1787,21 @@ static void
 game_new_cb (AisleriotGame *game,
              AisleriotWindow *window)
 {
+#if defined(MAEMO_TOOLBAR_BANNER) || !defined(HAVE_HILDON)
   AisleriotWindowPrivate *priv = window->priv;
+#endif
 
   update_statistics_display (window);
 
 #ifdef HAVE_HILDON
+#ifdef MAEMO_TOOLBAR_BANNER
   gtk_label_set_text (priv->game_message_label, NULL);
+#endif /* MAEMO_TOOLBAR_BANNER */
 #else
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
-#endif
+#endif /* HAVE_HILDON */
 }
 
 static void
@@ -1898,12 +1811,17 @@ game_statusbar_message_cb (AisleriotGame *game,
 {
   AisleriotWindowPrivate *priv = window->priv;
 #ifdef HAVE_HILDON
+#ifdef MAEMO_TOOLBAR_BANNER
   gtk_label_set_text (priv->game_message_label, message);
+#endif
 
   /* The banners are annoying, but until I get a better idea, use them
    * when the toolbar is hidden.
    */
-  if (!priv->toolbar_visible) {
+#ifdef MAEMO_TOOLBAR_BANNER
+  if (!priv->toolbar_visible)
+#endif
+  {
     guint hash;
 
     if (message != NULL && message[0] != '\0') {
@@ -2024,7 +1942,7 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
-#if defined(HAVE_CLUTTER) || (defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
 
 static void
 settings_changed_cb (GtkSettings *settings,
@@ -2050,14 +1968,14 @@ settings_changed_cb (GtkSettings *settings,
   }
 #endif /* HAVE_CLUTTER */
 
-#if defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0)
+#if defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0) {
     g_object_get (settings, "gtk-enable-event-sounds", &enabled, NULL);
 
     action = gtk_action_group_get_action (priv->action_group, "Sound");
     gtk_action_set_visible (action, enabled);
   }
-#endif /* HAVE_CANBERRA_GTK && GTK >= 2.14 */
+#endif /* ENABLE_SOUND && GTK >= 2.14 */
 }
 
 static void
@@ -2089,13 +2007,74 @@ screen_changed_cb (GtkWidget *widget,
   g_signal_connect (settings, "notify::gtk-enable-animations",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
-#if defined (HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0)
+#if defined (ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   g_signal_connect (settings, "notify::gtk-enable-event-sounds",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
 }
 
-#endif /* HAVE_CLUTTER || HAVE_CANBERRA_GTK */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
+
+/*
+ * aisleriot_window_set_freecell_mode:
+ * @window:
+ *
+ * Sets @window to FreeCell mode. In FreeCell mode,
+ * the window is using the FreeCell variation, and doesn't allow
+ * changing the game type.
+ */
+static void
+aisleriot_window_set_freecell_mode (AisleriotWindow *window,
+                                    gboolean freecell_mode)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  GtkAction *action;
+
+  priv->freecell_mode = freecell_mode != FALSE;
+
+  if (freecell_mode) {
+    /* Inhibit game changing */
+    action = gtk_action_group_get_action (priv->action_group, "Select");
+    gtk_action_set_visible (action, FALSE);
+    action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
+    gtk_action_set_visible (action, FALSE);
+
+    gtk_window_set_title (GTK_WINDOW (window), _("Freecell Solitaire"));
+  } else {
+    gtk_window_set_title (GTK_WINDOW (window), _("AisleRiot"));
+  }
+}
+
+#ifdef HAVE_CLUTTER
+
+static void
+board_cursor_cb (AisleriotBoard *board,
+                 int cursor_type,
+                 ArClutterEmbed *embed)
+{
+  ar_clutter_embed_set_cursor (embed, (ArCursorType) cursor_type);
+}
+
+static void
+board_error_bell_cb (AisleriotBoard *board,
+                     ArClutterEmbed *embed)
+{
+#if GTK_CHECK_VERSION (2, 12, 0) || (defined (HAVE_HILDON) && !defined(HAVE_MAEMO_3))
+  gtk_widget_error_bell (GTK_WIDGET (embed));
+#endif
+}
+
+static void
+embed_size_allocate_cb (ArClutterEmbed *embed,
+                        GtkAllocation *allocation,
+                        AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  clutter_actor_set_size (priv->board_actor, allocation->width, allocation->height);
+}
+
+#endif /* HAVE_CLUTTER */
 
 /* Class implementation */
 
@@ -2148,6 +2127,8 @@ aisleriot_window_state_event (GtkWidget *widget,
 
     set_fullscreen_actions (window, is_fullscreen);
 
+    set_fullscreen_button_active (window);
+
 #ifndef HAVE_HILDON
 #if GTK_CHECK_VERSION (2, 11, 0)
     gtk_statusbar_set_has_resize_grip (priv->statusbar, !is_maximised && !is_fullscreen);
@@ -2249,13 +2230,13 @@ aisleriot_window_init (AisleriotWindow *window)
 #endif /* ENABLE_CARD_THEMES_INSTALLER */
 
     /* Toolbar-only actions */
-#ifndef HAVE_MAEMO
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
     { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
       G_CALLBACK (leave_fullscreen_cb) },
+#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifndef HAVE_HILDON
     { "ThemeMenu", NULL, N_("_Card Style"), NULL, NULL, NULL },
 #endif /* !HAVE_HILDON */
-#endif /* !HAVE_MAEMO */
 
 #ifdef ENABLE_DEBUG_UI
     /* Debug UI actions */
@@ -2321,7 +2302,7 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (animations_toggle_cb),
       FALSE /* not active by default */ },
 #endif /* HAVE_CLUTTER */
-#ifdef ENABLE_DEBUG_UI
+#if defined(ENABLE_DEBUG_UI) && !defined(HAVE_CLUTTER)
     { "DebugPixbufDrawing", NULL, "_Pixbuf drawing", NULL, NULL,
       G_CALLBACK (debug_pixbuf_drawing_cb),
       FALSE },
@@ -2340,9 +2321,9 @@ aisleriot_window_init (AisleriotWindow *window)
 #ifdef HAVE_HILDON
     "AccelUndoMove",
 #endif /* HAVE_HILDON */
-#ifndef HAVE_MAEMO
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
     "LeaveFullscreen",
-#endif /* !HAVE_MAEMO */
+#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
   };
 
   static const char ui_description[] =
@@ -2385,7 +2366,9 @@ aisleriot_window_init (AisleriotWindow *window)
 #ifdef ENABLE_DEBUG_UI
         "<menu action='DebugMenu'>"
           "<menuitem action='DebugChooseSeed'/>"
+#ifndef HAVE_CLUTTER
           "<menuitem action='DebugPixbufDrawing'/>"
+#endif /* !HAVE_CLUTTER */
         "</menu>"
 #endif /* ENABLE_DEBUG_UI */
         "<menuitem action='CloseWindow'/>"
@@ -2462,10 +2445,10 @@ aisleriot_window_init (AisleriotWindow *window)
         "<toolitem action='RedoMove'/>"
         "<toolitem action='Deal'/>"
         "<toolitem action='Hint'/>"
-#ifndef HAVE_MAEMO
+#if !defined(HAVE_MAEMO) || !defined(MAEMO_TOOLBAR_BANNER)
         "<separator name='LeaveFullscreenSep' expand='true'/>"
         "<toolitem action='LeaveFullscreen'/>"
-#endif
+#endif /* !HAVE_MAEMO || !MAEMO_TOOLBAR_BANNER */
 #ifdef ENABLE_DEBUG_UI
         "<toolitem action='DebugGameFirst'/>"
         "<toolitem action='DebugGamePrev'/>"
@@ -2488,26 +2471,35 @@ aisleriot_window_init (AisleriotWindow *window)
   char *theme_name;
   GamesCardTheme *theme;
   guint i;
-#ifdef HAVE_HILDON
-  GtkToolItem *tool_item;
-#else
-  const char *env;
+#ifndef HAVE_HILDON
   GtkStatusbar *statusbar;
-  GtkWidget *statusbar_hbox,*label, *time_box;
+  GtkWidget *statusbar_hbox, *label, *time_box;
+#endif
+#ifdef HAVE_CLUTTER
+  ClutterContainer *stage;
 #endif
 
   g_assert (G_N_ELEMENTS (names) == LAST_ACTION);
 
   priv = window->priv = AISLERIOT_WINDOW_GET_PRIVATE (window);
 
+  priv->fullscreen = FALSE;
+
   priv->game = aisleriot_game_new ();
 
+#ifndef HAVE_CLUTTER
+
 #ifdef HAVE_HILDON
   priv->use_pixbuf_drawing = FALSE;
 #else
-  /* Default to pixbuf drawing */
+{
+  const char *env;
+
   env = g_getenv ("AISLERIOT_PIXBUF_DRAWING");
+
+  /* Default to pixbuf drawing */
   priv->use_pixbuf_drawing = env == NULL || g_ascii_strtoull (env, NULL, 10) != 0;
+}
 #endif /* HAVE_HILDON */
 
 #ifdef GNOME_ENABLE_DEBUG
@@ -2517,11 +2509,41 @@ aisleriot_window_init (AisleriotWindow *window)
     g_print ("Using pixmap drawing method\n");
 #endif /* GNOME_ENABLE_DEBUG */
 
+#endif /* !HAVE_CLUTTER */
+
   priv->theme_manager = games_card_themes_new ();
 
+#ifdef HAVE_CLUTTER
+  priv->board_style = ar_style_new ();
+  priv->board = ar_clutter_embed_new (priv->board_style);
+
+  priv->baize_actor = aisleriot_baize_new ();
+
+  stage = CLUTTER_CONTAINER (gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->board)));
+  clutter_container_add (stage, priv->baize_actor, NULL);
+  /* FIXMEchpe: how to ensure this is ALWAYS the lowest actor? */
+  clutter_actor_lower_bottom (priv->baize_actor);
+
+  priv->board_actor = aisleriot_board_new (priv->board_style, priv->game);
+  clutter_container_add (stage, priv->board_actor, NULL);
+
+  /* FIXMEchpe */
+  clutter_stage_set_key_focus (CLUTTER_STAGE (stage), priv->board_actor);
+
+  g_signal_connect_after (priv->board, "size-allocate",
+                          G_CALLBACK (embed_size_allocate_cb), window);
+
+  g_signal_connect (priv->board_actor, "request-cursor",
+                    G_CALLBACK (board_cursor_cb), priv->board);
+  g_signal_connect (priv->board_actor, "error-bell",
+                    G_CALLBACK (board_error_bell_cb), priv->board);
+
+  /* FIXMEchpe: unref baize & board_actor here? */
+#else
   priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->game));
 
   aisleriot_board_set_pixbuf_drawing (priv->board, priv->use_pixbuf_drawing);
+#endif /* HAVE_CLUTTER */
 
   theme_name = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
   theme = games_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
@@ -2577,16 +2599,31 @@ aisleriot_window_init (AisleriotWindow *window)
   gtk_statusbar_set_has_resize_grip (priv->statusbar, FALSE);
 #endif
 
+#if GTK_CHECK_VERSION (2, 19, 1)
+  statusbar_hbox = gtk_statusbar_get_message_area (statusbar);
+  gtk_box_set_spacing (GTK_BOX (statusbar_hbox), 24);
+#else
+{
+  GtkWidget *statusbar_label;
+  GtkContainer *statusbar_frame;
+  GList *list;
+
   /* Widget surgery: move the statusbar's label into a hbox
    * which we put in the statusbar's frame instead.
    */
   statusbar_hbox = gtk_hbox_new (FALSE, 24);
-  g_object_ref (statusbar->label);
-  gtk_container_remove (GTK_CONTAINER (statusbar->frame), statusbar->label);
-  gtk_box_pack_start (GTK_BOX (statusbar_hbox), statusbar->label, TRUE, TRUE, 0);
-  g_object_unref (statusbar->label);
-  gtk_container_add (GTK_CONTAINER (statusbar->frame), statusbar_hbox);
+  list = gtk_container_get_children (GTK_CONTAINER (statusbar));
+  statusbar_frame = GTK_CONTAINER (list->data);
+  g_list_free (list);
+  statusbar_label = gtk_bin_get_child (GTK_BIN (statusbar_frame));
+  g_object_ref (statusbar_label);
+  gtk_container_remove (statusbar_frame, statusbar_label);
+  gtk_box_pack_start (GTK_BOX (statusbar_hbox), statusbar_label, TRUE, TRUE, 0);
+  g_object_unref (statusbar_label);
+  gtk_container_add (statusbar_frame, statusbar_hbox);
   gtk_widget_show (statusbar_hbox);
+}
+#endif /* GTK+ >= 2.19.1 */
 
   /* Score */
   priv->score_box = gtk_hbox_new (12, FALSE);
@@ -2622,7 +2659,10 @@ aisleriot_window_init (AisleriotWindow *window)
   priv->main_menu = gtk_ui_manager_get_widget (priv->ui_manager, MAIN_MENU_UI_PATH);
   priv->toolbar = gtk_ui_manager_get_widget (priv->ui_manager, TOOLBAR_UI_PATH);
 
-#ifdef HAVE_HILDON
+#ifdef MAEMO_TOOLBAR_BANNER
+{
+  GtkToolItem *tool_item;
+
   /* It's a bit ugly to mess with the toolbar like this,
    * but it's simpler than to implement a dedicated action
    * which creates a tool item which ellipsises its label...
@@ -2637,7 +2677,8 @@ aisleriot_window_init (AisleriotWindow *window)
 
   gtk_toolbar_insert (GTK_TOOLBAR (priv->toolbar), tool_item, -1);
   gtk_widget_show_all (GTK_WIDGET (tool_item));
-#endif
+}
+#endif /* MAEMO_TOOLBAR_BANNER */
 
   /* Defer building the card themes menu until its parent's menu is opened */
 #ifdef HAVE_HILDON
@@ -2716,12 +2757,12 @@ aisleriot_window_init (AisleriotWindow *window)
 
 #endif /* HAVE_CLUTTER */
 
-#if defined(HAVE_CLUTTER) || (defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
   /* Set the action visibility and listen for animation and sound mode changes */
   screen_changed_cb (GTK_WIDGET (window), NULL, window);
   g_signal_connect (window, "screen-changed",
                     G_CALLBACK (screen_changed_cb), window);
-#endif /* HAVE_CLUTTER || HAVE_CANBERRA_GTK */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
 
   /* Now set up the widgets */
   main_vbox = gtk_vbox_new (FALSE, 0);
@@ -2796,18 +2837,18 @@ aisleriot_window_init (AisleriotWindow *window)
                     G_CALLBACK (sync_window_topmost_cb), NULL);
 #endif
 
-#ifdef ENABLE_DEBUG_UI
+#if defined(ENABLE_DEBUG_UI) && !defined(HAVE_CLUTTER)
   action = gtk_action_group_get_action (priv->action_group, "DebugPixbufDrawing");
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), priv->use_pixbuf_drawing);
-#endif /* ENABLE_DEBUG_UI */
+#endif /* ENABLE_DEBUG_UI && !HAVE_CLUTTER */
 }
 
 static void
-aisleriot_window_finalize (GObject *object)
+aisleriot_window_dispose (GObject *object)
 {
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
-
+  
 #ifdef HAVE_CLUTTER
   g_signal_handlers_disconnect_by_func (gtk_widget_get_settings (GTK_WIDGET (window)),
                                         G_CALLBACK (settings_changed_cb),
@@ -2832,6 +2873,31 @@ aisleriot_window_finalize (GObject *object)
     gtk_widget_destroy (GTK_WIDGET (priv->stats_dialog));
     g_assert (priv->stats_dialog == NULL);
   }
+  
+#ifdef LEAVE_FULLSCREEN_BUTTON
+  if (priv->fullscreen_button != NULL) {
+    gtk_widget_destroy (GTK_WIDGET (priv->fullscreen_button));
+    priv->fullscreen_button = NULL;
+  }
+#endif
+
+  if (priv->load_idle_id != 0) {
+    g_source_remove (priv->load_idle_id);
+    priv->load_idle_id = 0;
+  }
+
+  G_OBJECT_CLASS (aisleriot_window_parent_class)->dispose (object);
+}
+
+static void
+aisleriot_window_finalize (GObject *object)
+{
+  AisleriotWindow *window = AISLERIOT_WINDOW (object);
+  AisleriotWindowPrivate *priv = window->priv;
+
+#ifdef HAVE_CLUTTER
+  g_object_unref (priv->board_style);
+#endif /* HAVE_CLUTTER */
 
   if (priv->theme) {
     g_object_unref (priv->theme);
@@ -2844,20 +2910,35 @@ aisleriot_window_finalize (GObject *object)
                                         0, 0, NULL, NULL, window);
   g_object_unref (priv->game);
 
-  if (priv->load_idle_id != 0) {
-    g_source_remove (priv->load_idle_id);
-  }
-
   G_OBJECT_CLASS (aisleriot_window_parent_class)->finalize (object);
 }
 
+static void
+aisleriot_window_set_property (GObject      *object,
+                               guint         property_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
+{
+  AisleriotWindow *window = AISLERIOT_WINDOW (object);
+
+  switch (property_id) {
+    case PROP_FREECELL_MODE:
+      aisleriot_window_set_freecell_mode (window, g_value_get_boolean (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+  }
+}
 static void
 aisleriot_window_class_init (AisleriotWindowClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  gobject_class->dispose = aisleriot_window_dispose;
   gobject_class->finalize = aisleriot_window_finalize;
+  gobject_class->set_property = aisleriot_window_set_property;
 
   widget_class->window_state_event = aisleriot_window_state_event;
 #if GTK_CHECK_VERSION (2, 10, 0)
@@ -2865,6 +2946,20 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
 #endif
 
   g_type_class_add_private (gobject_class, sizeof (AisleriotWindowPrivate));
+
+  /**
+   * AisleriotWindow:freecell-mode:
+   *
+   * Whether the window is in freecell mode.
+   */
+  g_object_class_install_property
+    (gobject_class,
+     PROP_FREECELL_MODE,
+     g_param_spec_boolean ("freecell-mode", NULL, NULL,
+                           FALSE,
+                           G_PARAM_WRITABLE |
+                           G_PARAM_CONSTRUCT_ONLY |
+                           G_PARAM_STATIC_STRINGS));
 }
 
 /* public API */
@@ -2875,36 +2970,13 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
  * Returns: a new #AisleriotWindow
  */
 GtkWidget *
-aisleriot_window_new (void)
+aisleriot_window_new (gboolean freecell_mode)
 {
   return g_object_new (AISLERIOT_TYPE_WINDOW,
-                       "title", _("AisleRiot"),
+                       "freecell-mode", freecell_mode,
                        NULL);
 }
 
-/**
- * aisleriot_window_set_freecell_mode:
- * @window:
- *
- * Sets @window to FreeCell mode. In FreeCell mode,
- * the window is using the FreeCell variation, and doesn't allow
- * changing the game type.
- */
-void
-aisleriot_window_set_freecell_mode (AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkAction *action;
-
-  priv->freecell_mode = TRUE;
-
-  /* Inhibit game changing */
-  action = gtk_action_group_get_action (priv->action_group, "Select");
-  gtk_action_set_visible (action, FALSE);
-  action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
-  gtk_action_set_visible (action, FALSE);
-}
-
 typedef struct {
   AisleriotWindow *window;
   char *game_file;
